<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Models\User;
use App\Providers\RouteServiceProvider;
use Illuminate\Auth\Events\Registered;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Validation\Rules;
use App\Notifications\WelcomeMailNotification;

class RegisteredUserController extends Controller
{
    public function create()
    {
        return view('user.signup');
    }

    public function store(Request $request)
    {
        // Run validation
        $validator = \Validator::make($request->all(), [
            'first_name' => ['required', 'string', 'max:255'],
            'middle_name' => ['nullable', 'string', 'max:255'],
            'last_name' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
            'password' => ['required'],
            'phone' => ['required', 'max:255', 'unique:users'],
        ]);

        // Check for validation errors
        if ($validator->fails()) {
            $errorMessages = [];
            if ($validator->errors()->has('phone') && $validator->errors()->has('email')) {
                $errorMessages[] = 'This phone number and email have already been registered. Please <a href="/login">log in</a>.';
            } else {
                if ($validator->errors()->has('phone')) {
                    $errorMessages[] = 'This phone number has already been registered. Please <a href="/login">log in</a>.';
                }
                if ($validator->errors()->has('email')) {
                    $errorMessages[] = 'This email has already been registered. Please <a href="/login">log in</a>.';
                }
            }
            return redirect()->back()->withErrors(['custom_error' => implode(' ', $errorMessages)]);
        }

        // Generate verification code
        $verification_code = rand(100000, 999999);

        // Create user
        $user = User::create([
            'first_name' => $request->first_name,
            'middle_name' => $request->middle_name,
            'last_name' => $request->last_name,
            'email' => $request->email,
            'password' => Hash::make($request->password),
            'phone' => $request->phone,
            'verification_code' => $verification_code,
        ]);

        try {
            // Send verification code via email
            $this->sendVerificationCode($user->email, $verification_code);
            event(new Registered($user));
            Auth::login($user);

            // Send welcome email
            $user->notify(new WelcomeMailNotification($user));

            return redirect()->route('verify')->with('success', "Registration successful. A verification code has been sent to your email.");
        } catch (\Exception $e) {
            return redirect()->back()->with('error', 'Error occurred: ' . $e->getMessage());
        }
    }

    public function showVerificationForm()
{
    // Check if the user is authenticated and verified
    if (auth()->check() && auth()->user()->is_verified === 1) {
        // Redirect to dashboard if the user is already verified
        return redirect()->route('dashboard');
    }

    // Otherwise, show the OTP verification form
    return view('user.otp');
}


    public function verifyCode(Request $request)
    {
        $request->validate([
            'verification_code' => ['required', 'array', 'size:6'],
            'verification_code.*' => ['required', 'string', 'size:1'],
        ]);

        $verification_code = implode('', $request->verification_code);
        $user = Auth::user();

        if ($verification_code == $user->verification_code) {
            $user->is_verified = true;
            $user->verification_code = null;
            $user->save();

            $user->notify(new WelcomeMailNotification($user));

            return redirect()->route('princeupdate')->with('success', 'Verification successful. Welcome to the princeupdate');
        } else {
            return redirect()->route('verify')->withErrors(['verification_code' => 'The verification code is incorrect.']);
        }
    }

    // Send OTP via email
  private function sendVerificationCode($email, $code)
{
    Mail::raw("Your verification code is: $code", function ($message) use ($email) {
        $message->to($email)
                ->subject('Your Verification Code');
    });
}


    public function resendCode()
    {
        $user = Auth::user();
        
        if (!$user || $user->is_verified) {
            return redirect()->route('verify')->with('error', 'You are either already verified or not logged in.');
        }

        $verification_code = rand(100000, 999999);
        $user->verification_code = $verification_code;
        $user->save();

        try {
            // Resend verification code via email
            $this->sendVerificationCode($user->email, $verification_code);
            return redirect()->route('verify')->with('success', 'A new verification code has been sent to your email.');
        } catch (\Exception $e) {
            return redirect()->route('verify')->with('error', 'There was an issue sending the new verification code. Please try again.');
        }
    }
}
