<?php
namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Http;
use thiagoalessio\TesseractOCR\TesseractOCR;

class IDVerificationController extends Controller
{
    private $apiKey;
    private $model;
    private $url;

    public function __construct()
    {
        $this->apiKey = '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';
        $this->model = 'llama-3.1-8b-instant';
        $this->url = "https://api.groq.com/openai/v1/chat/completions";
    }

    public function verify(Request $request)
    {
        // Clear only the AI chat history from the session
        $request->session()->forget('ai_chat_history');

        // Store the uploaded files temporarily
        $frontPath = $request->file('front_id')->store('temp');
        $backPath = $request->file('back_id')->store('temp');

        // Extract text from the images using Tesseract OCR
        $ocrResultFront = (new TesseractOCR(storage_path('app/' . $frontPath)))->run();
        $ocrResultBack = (new TesseractOCR(storage_path('app/' . $backPath)))->run();

        // Delete the temporary files
        Storage::delete([$frontPath, $backPath]);

        // Perform AI-based validation
        $frontValidation = $this->validateWithAI($ocrResultFront, 'front');
        $backValidation = $this->validateWithAI($ocrResultBack, 'back');

        // Determine if the validation passed or failed
        $isFrontValid = $frontValidation['pass'];
        $isBackValid = $backValidation['pass'];

        // Check if AI detected non-ID card content and automatically fail
        if ($frontValidation['ai_response_contains_non_id'] || $backValidation['ai_response_contains_non_id']) {
            $isFrontValid = false;
            $isBackValid = false;
        }

        $response = [
            'status' => ($isFrontValid && $isBackValid) ? 'success' : 'error',
            'message' => ($isFrontValid && $isBackValid) ? 'ID verification successful' : 'ID verification failed',
            'front_validation' => $frontValidation,
            'back_validation' => $backValidation,
            'front_id_text' => $ocrResultFront,
            'back_id_text' => $ocrResultBack,
        ];

        return response()->json($response, $response['status'] === 'success' ? 200 : 400);
    }

    private function validateWithAI($text, $side)
    {
        $prompt = $this->generatePrompt($text, $side);
    
        $response = Http::withHeaders([
            'Authorization' => 'Bearer ' . $this->apiKey,
            'Content-Type' => 'application/json',
        ])->post($this->url, [
            'model' => $this->model,
            'messages' => [
                ['role' => 'system', 'content' => 'You are an AI assistant that analyzes text from Kenyan ID cards.'],
                ['role' => 'user', 'content' => $prompt],
            ],
            'temperature' => 0.1,
        ]);
    
        if ($response->successful()) {
            $result = json_decode($response->body(), true);
            $aiResponse = $result['choices'][0]['message']['content'];
            
            // Store AI response in session for future reference (if needed)
            session()->put('ai_chat_history', $aiResponse);
            
            return $this->parseAIResponse($aiResponse, $side);
        } else {
            return [
                'pass' => false,
                'message' => "AI validation failed for $side side",
                'ai_response' => "Failed to get AI response",
            ];
        }
    }

    private function parseAIResponse($aiResponse, $side)
    {
        $lines = explode("\n", $aiResponse);
        $foundInfo = [];
        $missingInfo = [];
        $containsNonIdContent = false;
        $containsTyposOrMisspellings = false;

        foreach ($lines as $line) {
            if (strpos($line, 'Present:') !== false) {
                $foundInfo = array_map('trim', explode(',', substr($line, strpos($line, ':') + 1)));
            } elseif (strpos($line, 'Missing:') !== false) {
                $missingInfo = array_map('trim', explode(',', substr($line, strpos($line, ':') + 1)));
            }

            // Check if the response contains an indication that the text is not from a Kenyan ID
            if (strpos(strtolower($line), 'not from a kenyan id card') !== false) {
                $containsNonIdContent = true;
            }

            // Check if the response mentions possible typos or misspellings
            if (strpos(strtolower($line), 'typo') !== false || strpos(strtolower($line), 'misspelling') !== false || strpos(strtolower($line), 'unclear') !== false) {
                $containsTyposOrMisspellings = true;
            }
        }

        // If non-ID content is detected, fail the validation.
        // If there are typos/misspellings but the AI suspects it’s an ID, pass the validation.
        $pass = $containsNonIdContent ? false : (count($foundInfo) > 0 || $containsTyposOrMisspellings);

        return [
            'pass' => $pass,
            'message' => $pass ? "$side side validation passed" : "$side side validation failed",
            'ai_response' => $aiResponse, // Include the full AI response for reference
            'found_information' => $foundInfo,
            'missing_information' => $missingInfo,
            'ai_response_contains_non_id' => $containsNonIdContent,
            'ai_response_contains_typos_or_misspellings' => $containsTyposOrMisspellings,
        ];
    }

    private function generatePrompt($text, $side)
    {
        if ($side === 'front') {
            return "Analyze the following text from the front of a Kenyan ID card. Determine if it contains the following information: Republic of Kenya, ID Number, Full Names, Date of Birth, Sex, District of Birth, Date of Issue. List which information is present and which is missing. Text: $text";
        } else {
            return "Analyze the following text from the back of a Kenyan ID card. Determine if it contains the following information: District, Division, Location, Sub-Location. List which information is present and which is missing. Text: $text";
        }
    }
}
