import React, { useState, useEffect } from 'react';
import { useVoice } from "@humeai/voice-react";
import { AnimatePresence, motion } from "framer-motion";
import { Button } from "./ui/button";
import { Phone } from "lucide-react";

// Define the prop types for CallScreenOverlay
interface CallScreenOverlayProps {
  onAnimationComplete: () => void;
}

const CallScreenOverlay: React.FC<CallScreenOverlayProps> = ({ onAnimationComplete }) => {
  const [ringCount, setRingCount] = useState(0);

  useEffect(() => {
    if (ringCount < 3) {
      const timer = setTimeout(() => {
        setRingCount(ringCount + 1);
      }, 1000);
      return () => clearTimeout(timer);
    } else {
      onAnimationComplete();
    }
  }, [ringCount, onAnimationComplete]);

  return (
    <motion.div
      className="fixed inset-0 bg-black bg-opacity-50 flex items-center justify-center"
      initial={{ opacity: 0 }}
      animate={{ opacity: 1 }}
      exit={{ opacity: 0 }}
    >
      <motion.div
        className="bg-white rounded-3xl w-64 h-128 flex flex-col items-center justify-between p-6 shadow-lg"
        initial={{ scale: 0.5 }}
        animate={{ scale: 1 }}
        exit={{ scale: 0.5 }}
      >
        <div className="flex flex-col items-center space-y-4">
          <div className="w-24 h-24 bg-gray-300 rounded-full flex items-center justify-center">
            <Phone className="h-12 w-12 text-gray-600" />
          </div>
          <h2 className="text-2xl font-semibold">Calling...</h2>
          <div className="flex space-x-2">
            {[0, 1, 2].map((i) => (
              <motion.div
                key={i}
                className="w-2 h-2 bg-green-500 rounded-full"
                animate={{
                  scale: [1, 1.5, 1],
                  opacity: [1, 0.5, 1],
                }}
                transition={{
                  duration: 1,
                  repeat: Infinity,
                  delay: i * 0.2,
                }}
              />
            ))}
          </div>
        </div>
      </motion.div>
    </motion.div>
  );
};

export default function StartCall() {
  const { status, connect } = useVoice();
  const [showOverlay, setShowOverlay] = useState(false);
  const [isAnimating, setIsAnimating] = useState(false);

  const handleStartCall = () => {
    setShowOverlay(true);
    setIsAnimating(true);
  };

  const handleAnimationComplete = () => {
    setIsAnimating(false);
    setShowOverlay(false);
    connect()
      .then(() => {})
      .catch(() => {})
      .finally(() => {});
  };

  return (
    <>
      <AnimatePresence>
        {status.value !== "connected" && !isAnimating && (
          <motion.div
            className="fixed inset-0 p-4 flex items-center justify-center bg-background"
            initial={{ opacity: 0 }}
            animate={{ opacity: 1 }}
            exit={{ opacity: 0 }}
          >
            <motion.div
              initial={{ scale: 0.5 }}
              animate={{ scale: 1 }}
              exit={{ scale: 0.5 }}
            >
              <Button
                className="z-50 flex items-center gap-1.5"
                onClick={handleStartCall}
              >
                <span>
                  <Phone
                    className="size-4 opacity-50"
                    strokeWidth={2}
                    stroke="currentColor"
                  />
                </span>
                <span>Start Call</span>
              </Button>
            </motion.div>
          </motion.div>
        )}
      </AnimatePresence>
      
      <AnimatePresence>
        {showOverlay && (
          <CallScreenOverlay onAnimationComplete={handleAnimationComplete} />
        )}
      </AnimatePresence>
    </>
  );
}
